@@ -11,6 +11,7 @@ use swc_ecma_ast::*;
 use swc_ecma_transforms_base::pass::JsPass;
 use swc_ecma_utils::{
     default_constructor, ident::IdentLike, prepend, private_ident, quote_ident, ExprFactory, Id,
+    HANDLER,
 };
 use swc_ecma_visit::{
     as_folder, noop_visit_mut_type, noop_visit_type, Visit, VisitMut, VisitMutWith, VisitWith,
@@ -367,6 +368,15 @@ impl VisitMut for PrivateInObject {
                 });
             }
 
+            Expr::Member(MemberExpr {
+                span,
+                obj: _,
+                prop: MemberProp::PrivateName(PrivateName { .. }),
+            }) => {
+                let error = "Illegal access to private class property.";
+                HANDLER.with(|h| h.struct_span_err(*span, error).emit());
+            }
+
             _ => {}
         }
     }
